@@ -9,18 +9,9 @@ our @ISA = qw(Exporter);
 
 our @EXPORT_OK = qw(c2xs);
 
-our $VERSION = 0.14;
+our $VERSION = 0.15;
 
-sub c2xs {
-    eval {require "Inline/C.pm"};
-    if($@ || $Inline::C::VERSION < 0.44) {die "Need a functioning Inline::C (version 0.44 or later). $@"}
-    my $module = shift;
-    my $pkg = shift;
-
-    # Set the defaults for $build_dir and $config_options.
-    # (These will be overwritten by any supplied arguments.)
-    my ($build_dir, $config_options) = (
-       '.', 
+my $config_options =
        {
        'AUTOWRAP' => 0,
        'AUTO_INCLUDE' => '',
@@ -29,7 +20,17 @@ sub c2xs {
        'WRITE_MAKEFILE_PL' => 0,
        'WRITE_PM' => 0,
        }
-                                        );
+;        
+
+sub c2xs {
+    eval {require "Inline/C.pm"};
+    if($@ || $Inline::C::VERSION < 0.45) {die "Need a functioning Inline::C (version 0.45 or later). $@"}
+    my $module = shift;
+    my $pkg = shift;
+
+    # Set the default for $build_dir.
+    # (This will be overwritten by a supplied build_dir argument.)
+    my $build_dir = '.';
     
     if(@_) {
       if(ref($_[0]) eq "HASH") {
@@ -245,23 +246,86 @@ sub _build {
 
 sub _check_config_keys {
     for('AUTOWRAP', 'AUTO_INCLUDE', 'TYPEMAPS', 'LIBS', 'INC', 'VERSION', 'WRITE_MAKEFILE_PL',
-        'BUILD_NOISY', 'BOOT', 'MAKE', 'PREFIX', 'CCFLAGS', 'LD', 'LDDLFLAGS', 'MYEXTLIB', 
-        'OPTIMIZE', 'CC', 'USING', 'WRITE_PM', 'CODE', 'SRC_LOCATION')
+        'BUILD_NOISY', 'BOOT', 'EXPORT_ALL', 'EXPORT_OK_ALL', 'EXPORT_TAGS_ALL', 'MAKE',
+        'PREFIX', 'CCFLAGS', 'LD', 'LDDLFLAGS', 'MYEXTLIB', 'OPTIMIZE', 'CC', 'USING',
+        'WRITE_PM', 'CODE', 'SRC_LOCATION')
        {return 1 if $_ eq $_[0]} # it's a valid config option
     return 0;                    # it's an invalid config option
 }
 
 sub _write_pm {
     my $o = shift;
+    my $offset = 4;
+    my $max = 100;
+    my $length = $offset;
+
     open(WR, '>', $o->{API}{build_dir} . '/' . $o->{API}{modfname} . ".pm")
         or die "Couldn't create the .pm file: $!";
     print "Writing ", $o->{API}{modfname}, ".pm in the ", $o->{API}{build_dir}, " directory\n";
+    print WR "## This file generated by InlineX::C2XS (version ",
+             $InlineX::C2XS::VERSION, ") using Inline::C (version ", $Inline::C::VERSION, ")\n";
     print WR "package ", $o->{API}{module}, ";\nuse strict;\n\n";
     print WR "require Exporter;\n*import = \\&Exporter::import;\nrequire DynaLoader;\n\n";
     print WR "\$", $o->{API}{module}, "::VERSION = '", $o->{API}{version}, "';\n\n"; 
     print WR "DynaLoader::bootstrap ", $o->{API}{module}, " \$", $o->{API}{module}, "::VERSION;\n\n";
-    print WR "\@", $o->{API}{module}, "::EXPORT = ();\n";
-    print WR "\@", $o->{API}{module}, "::EXPORT_OK = ();\n\n";
+
+    unless($config_options->{EXPORT_ALL}) {
+      print WR "\@", $o->{API}{module}, "::EXPORT = ();\n";
+    }
+    else {
+      print WR "\@", $o->{API}{module}, "::EXPORT = qw(\n";
+      for(@{$o->{ILSM}{parser}{data}{functions}}) {
+        if ($_ =~ /^_/ && $_ !~ /^__/) {next} 
+        my $l = length($_);
+        if($length + $l > $max) {
+          print WR "\n", " " x $offset, "$_ ";
+          $length = $offset + $l + 1;
+        }
+        if($length == $offset) {print WR " " x $offset, "$_ "}
+        else {print WR "$_ " }
+        $length += $l + 1;
+      }
+      print WR "\n", " " x $offset, ");\n\n";
+      $length = $offset;
+    }
+
+    unless($config_options->{EXPORT_OK_ALL}) {
+      print WR "\@", $o->{API}{module}, "::EXPORT_OK = ();\n\n";
+    }
+    else {
+      print WR "\@", $o->{API}{module}, "::EXPORT_OK = qw(\n";
+      for(@{$o->{ILSM}{parser}{data}{functions}}) {
+        if ($_ =~ /^_/ && $_ !~ /^__/) {next} 
+        my $l = length($_);
+        if($length + $l > $max) {
+          print WR "\n", " " x $offset, "$_ ";
+          $length = $offset + $l + 1;
+        }
+        if($length == $offset) {print WR " " x $offset, "$_ "}
+        else {print WR "$_ " }
+        $length += $l + 1;
+      }
+      print WR "\n", " " x $offset, ");\n\n";
+      $length = $offset;
+    }
+
+    if($config_options->{EXPORT_TAGS_ALL}){
+      print WR "\%", $o->{API}{module}, "::EXPORT_TAGS = (", $config_options->{EXPORT_TAGS_ALL}, " => [qw(\n";
+      for(@{$o->{ILSM}{parser}{data}{functions}}) {
+        if ($_ =~ /^_/ && $_ !~ /^__/) {next} 
+        my $l = length($_);
+        if($length + $l > $max) {
+          print WR "\n", " " x $offset, "$_ ";
+          $length = $offset + $l + 1;
+        }
+        if($length == $offset) {print WR " " x $offset, "$_ "}
+        else {print WR "$_ " }
+        $length += $l + 1;
+      }
+      print WR "\n", " " x $offset, ")]);\n\n";
+      $length = $offset;
+    }
+
     print WR "sub dl_load_flags {0} # Prevent DynaLoader from complaining and croaking\n\n";
     print WR "1;\n";
     close(WR) or die "Couldn't close the .pm file after writing to it: $!";
@@ -386,7 +450,7 @@ InlineX::C2XS - Convert from Inline C code to XS.
 =head1 Recognised Hash Keys
 
  As regards the optional fourth argument to c2xs(), the following hash
- keys are recognised:
+ keys/values are recognised:
 
   AUTO_INCLUDE
    The value specified is automatically inserted into the generated XS
@@ -438,6 +502,32 @@ InlineX::C2XS - Convert from Inline C code to XS.
     CODE => 'void foo() {printf("Hello World\n";}' . "\n\n",
   ----
 
+  EXPORT_ALL
+   Makes no sense to use this unless WRITE_PM has been set.
+   Places all XSubs except those beginning with a *single* underscore (but not
+   multiple underscores) in @EXPORT in the generated .pm file. eg:
+
+    EXPORT_ALL => 1,
+  ----
+
+  EXPORT_OK_ALL
+   Makes no sense to use this unless WRITE_PM has been set.
+   Places all XSubs except those beginning with a *single* underscore (but not
+   multiple underscores) in @EXPORT_OK in the generated .pm file. eg:
+
+    EXPORT_OK_ALL => 1,
+  ----
+
+  EXPORT_TAGS_ALL
+   Makes no sense to use this unless WRITE_PM has been set.
+   In the generated .pm file, creates an EXPORT_TAGS tag named 'name'
+   (where 'name' is whatever you have specified), and places all XSubs except
+   those beginning with a *single* underscore (but not multiple underscores)
+   in 'name'. eg, the following creates and fills a tag named 'all':
+
+    EXPORT_TAGS_ALL => 'all',
+  ----
+
   INC
    The value specified is added to the includes search path. It makes
    sense to assign this key only when AUTOWRAP and/or WRITE_MAKEFILE_PL
@@ -543,8 +633,9 @@ InlineX::C2XS - Convert from Inline C code to XS.
    Set this to a true value if you want a .pm file to be generated.
    You'll also need to assign the 'VERSION' key appropriately. 
    Note that it's a fairly simplistic .pm file - no POD, no perl 
-   subroutines, no exported subs, no warnings - but it will allow 
-   the utilisation of all of the XSubs in the XS file. eg:
+   subroutines, no exported subs (unless EXPORT_ALL or EXPORT_OK_ALL
+   has been set), no warnings - but it will allow the utilisation of all of
+   the XSubs in the XS file. eg:
 
     WRITE_PM => 1,
   ----
